﻿
'------------------------------------------------------------------------------
'
'SMART ROI V1A
'
'BY:
'
'Matthew M. Batchelor
'President and Applications Engineer
'Alces Imaging and Automation, LLC
'2906 Blue Wind Court
'Houston, TX
'77084
'
'------------------------------------------------------------------------------
'
'This program allows its users to create an ROI within an IMAGE and then
'duplicate that ROI in another image or modify the ROI to contain the same
'amount of area.
'
'Due to roundoff caused by the integer nature of PIXELS, the AREA of the ROI
'will change slightly.
'
'------------------------------------------------------------------------------
'
'MODIFICATION LOG:
'-- V1A
'2018-01-02
'
'------------------------------------------------------------------------------

'IMPORT THE APPROPRIATE LIBRARIES
Imports MediaCy.IQL.Features

Public Class Smart_Roi_App_V1A

    Private Sub MyControl_AppLoading() Handles MyControl.AppLoading

       'MsgBox("AppLoading")

        MyControl_Load(Nothing,Nothing)

    End Sub

    Private Sub MyControl_AppUnloading() Handles MyControl.AppUnloading

        'MsgBox("AppLoading")

        'Turn off the timer
        timer1.Stop

    End Sub


    Private Sub MyControl_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyControl.Load

        ' Initialization code goes here

        'IF THERE ARE IMAGE DOCUMENTS OPEN
        If (ThisApplication.Documents.Count > 0) _
            Then

                'LEARN THE CALIBRATION FACTOR FOR THE CURRENT IMAGE
                Dim My_CalibFactorX As Double
                Dim My_CalibFactorY As Double
                If (ThisApplication.ActiveImage.SpatialCalibration IsNot Nothing) _
                    Then

                        My_CalibFactorX = _
                            ThisApplication.ActiveImage.SpatialCalibration.PixelSizeX

                        My_CalibFactorY = _
                            ThisApplication.ActiveImage.SpatialCalibration.PixelSizeY

                    Else

                        My_CalibFactorX = _
                            1

                        My_CalibFactorY = _
                            1

                    End If

                'UPDATE DIALOG BOX ELEMENTS
                textBox_RoiArea.Text = _
                    Format _
                    ( _
                    ( ( ThisApplication.Activeimage.Width * .5 ) * My_CalibFactorX ) * _
                    ( ( ThisApplication.Activeimage.Height * .5 ) * My_CalibFactorY ) _
                    , "0.0000"
                    )

                textBox_RoiCenterX.text = _
                    Format _
                    ( _
                    ThisApplication.Activeimage.Width * .5 * My_CalibFactorX _
                    , "0.0000"
                    )

                textBox_RoiCenterY.text = _
                    Format _
                    ( _
                    ThisApplication.Activeimage.Height * .5 * My_CalibFactorY _
                    , "0.0000"
                    )

                textBox_RoiWidth.text = _
                    Format _
                    ( _
                    ThisApplication.Activeimage.Width * .25 * My_CalibFactorX _
                    , "0.0000"
                    )

                textBox_RoiHeight.text = _
                    Format _
                    ( _
                    ThisApplication.Activeimage.Height * .25 * My_CalibFactorX _
                    , "0.0000"
                    )

            End If

        'Turn on the timer
        timer1.Start

    End Sub

    Protected Overrides Sub Finalize()
        ' Cleanup code goes here
    End Sub

    ' Provides access to underlying Control
    Public ReadOnly Property Control As System.Windows.Forms.UserControl
        Get
            Return MyControl
        End Get
    End Property

    Private Sub button_GetAreaFromRoi_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles button_GetAreaFromRoi.Click

        'IF THERE ARE DOCUMENTS OPEN
        If (ThisApplication.Documents.Count > 0) _
            Then

                'CONNECT WITH THE ROIS IN THE ACTIVE IMAGE
                Dim My_Roi As McRegions = _
                    ThisApplication.ActiveImage.Aoi

                'IF THERE ARE ROIS OPEN
                If (My_Roi.Count > 0) _
                    Then

                        'GET AREA FROM CURRENT ROI
                        textBox_RoiArea.Text = _
                            Trim ( Format ( My_Roi.mRgnArea.value(0) , "0.0000") )


                        'CALCULATE WIDTH AND HEIGHT FROM CURRENT ROI
                        textBox_RoiWidth.Text = _
                            Trim ( Format ( Sqr(My_Roi.mRgnArea.value(0)) , "0.0000") )
                        textBox_RoiHEIGHT.Text = _
                            Trim ( Format ( Sqr(My_Roi.mRgnArea.value(0)) , "0.0000") )

                    Else

                        'MESSAGE FOR USER
                        MsgBox _
                            ( _
                            "There is no ACTIVE ROI for me to work with.", _
                            vbCritical, _
                            "'button_GetAreaFromRoi_Click' says . . ." _
                            )

                    End If

            Else

                'MESSAGE FOR USER
                MsgBox _
                    ( _
                    "There is no ACTIVE IMAGE for me to work with.", _
                    vbCritical, _
                    "'button_GetAreaFromRoi_Click' says . . ." _
                    )

            End If

    End Sub

	Private Sub button_GetCenterFromRoi_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles button_GetCenterFromRoi.Click

        'IF THERE ARE DOCUMENTS OPEN
        If (ThisApplication.Documents.Count > 0) _
            Then

                'CONNECT WITH THE ROIS IN THE ACTIVE IMAGE
                'Dim My_Roi As McRegions = _
                Dim My_Roi As McRegions = _
                    ThisApplication.ActiveImage.Aoi

                'IF THERE ARE ROIS OPEN
                If (My_Roi.Count > 0) _
                    Then

                        'GET CENTER FROM CURRENT ROI
                        textBox_RoiCenterX.text = _
                            Trim ( Format ( My_Roi.mRgnCentroidX.value(0) , "0.0000") )

                        textBox_RoiCenterY.text = _
                            Trim ( Format ( My_Roi.mRgnCentroidY.value(0)  , "0.0000") )

                    Else

                        'MESSAGE FOR USER
                        MsgBox _
                            ( _
                            "There is no ACTIVE ROI for me to work with.", _
                            vbCritical, _
                            "'button_GetCenterFromRoi_Click' says . . ." _
                            )

                    End If

            Else

                'MESSAGE FOR USER
                MsgBox _
                    ( _
                    "There is no ACTIVE IMAGE for me to work with.", _
                    vbCritical, _
                    "'button_GetCenterFromRoi_Click' says . . ." _
                    )

            End If

    End Sub

    Private Sub button_CreateSmartRoi_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles button_CreateSmartRoi.Click

        'IF THERE ARE DOCUMENTS OPEN
        If (ThisApplication.Documents.Count > 0) _
            Then

                'CREATE AN ROI BASED ON VALUES IN DIALOG BOX
                SmartRoi_MakeRoi

            Else

                'MESSAGE FOR USER
                MsgBox _
                    ( _
                    "There is no ACTIVE IMAGE for me to work with.", _
                    vbCritical, _
                    "'button_CreateSmartRoi_Click' says . . ." _
                    )

            End If

    End Sub


    Private Sub button_WidthMinus_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles button_WidthMinus.Click

        'IF THERE ARE DOCUMENTS OPEN
        If (ThisApplication.Documents.Count = 0) _
            Then

                'MESSAGE FOR USER
                MsgBox _
                    ( _
                    "There is no ACTIVE IMAGE for me to work with.", _
                    vbCritical, _
                    "'SMART ROI V1A' says . . ." _
                    )

                Exit Sub

            End If

        'GET THE PERCENTAGE FROM THE RADIO BUTTONS
        Dim My_Percentage As Double
        If (radioButton_Fine.Checked = True) _
            Then
                My_Percentage = 00.1
            End If

        If (radioButton_Normal.Checked = True) _
            Then
                My_Percentage = 01.0
            End If

        If (radioButton_Coarse.Checked = True) _
            Then
                My_Percentage = 10.0
            End If

        'CALCULATE THE NEW WIDTH
        textBox_RoiWidth.TEXT = _
            Format _
            ( _
            ( Val(textBox_RoiWidth.TEXT) * ( 1.0 - ( My_Percentage / 100. ) ) ), _
            ,"0.0000" _
            )

        'CALCULATE THE NEW HEIGHT
        textBox_RoiHEIGHT.TEXT = _
            Format _
            ( _
            Val(textBox_RoiArea.TEXT) / _
            Val(textBox_RoiWidth.TEXT)
            ,"0.0000" _
            )

        'CREATE AN ROI BASED ON VALUES IN DIALOG BOX
        SmartRoi_MakeRoi

    End Sub

    Private Sub button_WidthPlus_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles button_WidthPlus.Click

        'IF THERE ARE DOCUMENTS OPEN
        If (ThisApplication.Documents.Count = 0) _
            Then

                'MESSAGE FOR USER
                MsgBox _
                    ( _
                    "There is no ACTIVE IMAGE for me to work with.", _
                    vbCritical, _
                    "'SMART ROI V1A' says . . ." _
                    )

                Exit Sub

            End If

        'GET THE PERCENTAGE FROM THE RADIO BUTTONS
        Dim My_Percentage As Double
        If (radioButton_Fine.Checked = True) _
            Then
                My_Percentage = 00.1
            End If

        If (radioButton_Normal.Checked = True) _
            Then
                My_Percentage = 01.0
            End If

        If (radioButton_Coarse.Checked = True) _
            Then
                My_Percentage = 10.0
            End If

        'CALCULATE THE NEW WIDTH
        textBox_RoiWidth.TEXT = _
            Format _
            ( _
            ( Val(textBox_RoiWidth.TEXT) / ( 1.0 - ( My_Percentage / 100. ) ) ), _
            ,"0.0000" _
            )

        'CALCULATE THE NEW HEIGHT
        textBox_RoiHEIGHT.TEXT = _
            Format _
            ( _
            Val(textBox_RoiArea.TEXT) / _
            Val(textBox_RoiWidth.TEXT)
            ,"0.0000" _
            )

        'CREATE AN ROI BASED ON VALUES IN DIALOG BOX
        SmartRoi_MakeRoi

    End Sub

    Private Sub button_HeightMinus_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles button_HeightMinus.Click

        'IF THERE ARE DOCUMENTS OPEN
        If (ThisApplication.Documents.Count = 0) _
            Then

                MsgBox _
                    ( _
                    "There is no ACTIVE IMAGE for me to work with.", _
                    vbCritical, _
                    "'SMART ROI V1A' says . . ." _
                    )

                Exit Sub

            End If

        'GET THE PERCENTAGE FROM THE RADIO BUTTONS
        Dim My_Percentage As Double
        If (radioButton_Fine.Checked = True) _
            Then
                My_Percentage = 00.1
            End If

        If (radioButton_Normal.Checked = True) _
            Then
                My_Percentage = 01.0
            End If

        If (radioButton_Coarse.Checked = True) _
            Then
                My_Percentage = 10.0
            End If

        'CALCULATE THE NEW HEIGHT
        textBox_RoiHeight.TEXT = _
            Format _
            ( _
            ( Val(textBox_RoiHeight.TEXT) * ( 1.0 - ( My_Percentage / 100. ) ) ), _
            ,"0.0000" _
            )

        'CALCULATE THE NEW WIDTH
        textBox_RoiWidth.TEXT = _
            Format _
            ( _
            Val(textBox_RoiArea.TEXT) / _
            Val(textBox_RoiHeight.TEXT)
            ,"0.0000" _
            )

        'CREATE AN ROI BASED ON VALUES IN DIALOG BOX
        SmartRoi_MakeRoi

    End Sub

    Private Sub button_HeightPlus_Click(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles button_HeightPlus.Click

        'IF THERE ARE DOCUMENTS OPEN
        If (ThisApplication.Documents.Count = 0) _
            Then

                MsgBox _
                    ( _
                    "There is no ACTIVE IMAGE for me to work with.", _
                    vbCritical, _
                    "'SMART ROI V1A' says . . ." _
                    )

                Exit Sub

            End If

        'GET THE PERCENTAGE FROM THE RADIO BUTTONS
        Dim My_Percentage As Double
        If (radioButton_Fine.Checked = True) _
            Then
                My_Percentage = 00.1
            End If

        If (radioButton_Normal.Checked = True) _
            Then
                My_Percentage = 01.0
            End If

        If (radioButton_Coarse.Checked = True) _
            Then
                My_Percentage = 10.0
            End If

        'CALCULATE THE NEW HEIGHT
        textBox_RoiHeight.TEXT = _
            Format _
            ( _
            ( Val(textBox_RoiHeight.TEXT) / ( 1.0 - ( My_Percentage / 100. ) ) ), _
            ,"0.0000" _
            )

        'CALCULATE THE NEW WIDTH
        textBox_RoiWidth.TEXT = _
            Format _
            ( _
            Val(textBox_RoiArea.TEXT) / _
            Val(textBox_RoiHeight.TEXT)
            ,"0.0000" _
            )

        'CREATE AN ROI BASED ON VALUES IN DIALOG BOX
        SmartRoi_MakeRoi

    End Sub

    Private Sub SmartRoi_MakeRoi()

        'IF THERE ARE DOCUMENTS OPEN
        If (ThisApplication.Documents.Count = 0) _
            Then

                MsgBox _
                    ( _
                    "There is no ACTIVE IMAGE for me to work with.", _
                    vbCritical, _
                    "'button_CreateSmartRoi_Click' says . . ." _
                    )

                Exit Sub

            End If

        'CONNECT WITH THE ACTIVE IMAGE
        Dim docA, imageA

        With Application.DocumentCommands.Active(Nothing)
            .Run(docA)
        End With

        With Application.DocumentCommands.ActiveImage(Nothing)
            .Run(imageA)
        End With

        'DELETE ALL ROIS FROM ACTIVE IMAGE
        With [Select].RoiCommands.DeleteAll(Nothing)
            .Run(docA)
        End With

        'LEARN THE CALIBRATION FACTOR FOR THE CURRENT IMAGE
        Dim My_CalibFactorX As Double
        Dim My_CalibFactorY As Double
        If (ThisApplication.ActiveImage.SpatialCalibration IsNot Nothing) _
            Then

                My_CalibFactorX = _
                    ThisApplication.ActiveImage.SpatialCalibration.PixelSizeX

                My_CalibFactorY = _
                    ThisApplication.ActiveImage.SpatialCalibration.PixelSizeY

            Else

                My_CalibFactorX = _
                    1

                My_CalibFactorY = _
                    1

            End If

        'CALCULATE THE UL AND LR CORNERS OF THE ROI FROM THE VALUES IN THE DIALOG BOX

        Dim MY_ULX, MY_ULY As Double

        MY_ULX = _
            ( ( Val(textBox_RoiCenterX.Text) - .5 * Val(textBox_RoiWidth.Text) ) / My_CalibFactorX ) + .5

        MY_ULY = _
            ( ( Val(textBox_RoiCenterY.Text) - .5 * Val(textBox_RoiHEIGHT.Text) ) / My_CalibFactorY ) + .5

        Dim MY_LRX, MY_LRY As Double

        MY_LRX = _
            ( ( Val(textBox_RoiCenterX.Text) + .5 * Val(textBox_RoiWidth.Text) ) / My_CalibFactorX ) - .5

        MY_LRY = _
            ( ( Val(textBox_RoiCentery.Text) + .5 * Val(textBox_RoiHEIGHT.Text) ) / My_CalibFactorY ) - .5

        'CREATE THE STANDARD ROI USING THE CENTER, WIDTH, AND HEIGHT FROM THE DIALOG BOX
        With [Select].RoiCommands.Add(Nothing)
            .ROIType = Features.ROI.ROITypes.Rectangle
            .Points = New System.Collections.Generic.List(Of System.Drawing.PointF)
            .Points.Add(New System.Drawing.PointF(MY_ULX,MY_ULY))
            .Points.Add(New System.Drawing.PointF(MY_LRX,MY_LRY))
            .Angle = 0R
            .Run(imageA)
        End With

        'CONNECT WITH THE ROIS IN THE ACTIVE IMAGE
        Dim My_Roi As McRegions = _
            ThisApplication.ActiveImage.Aoi

        'GET ROI SPECIFICATIONS FROM CURRENT ROI
        textBox_RoiArea.Text = _
            Trim ( Format ( My_Roi.mRgnArea.mean, "0.0000") )

        textBox_RoiCenterX.text = _
            Trim ( Format ( My_Roi.mRgnCentroidX.mean , "0.0000") )

        textBox_RoiCenterY.text = _
            Trim ( Format ( My_Roi.mRgnCentroidY.mean  , "0.0000") )

        textBox_RoiWidth.text = _
            Trim ( Format ( My_Roi.mRgnBoundingBoxWidth.mean , "0.0000") )

        textBox_RoiHeight.text = _
            Trim ( Format ( My_Roi.mRgnBoundingBoxheight.mean , "0.0000") )

    End Sub

    Private Sub Update_Dialog

        'If there is an open document
        If (ThisApplication.Documents.Count > 0) _
            Then

                'Image Calibration
                If (ThisApplication.ActiveImage.SpatialCalibration IsNot Nothing) _
                    Then

                        label_CalibUnitsArea.Text = _
                            "(sq. " & _
                            ThisApplication.ActiveImage.SpatialCalibration.UnitAbbrev & _
                            ".)

                        label_CalibUnitsCenterX.Text = _
                            "(" & _
                            ThisApplication.ActiveImage.SpatialCalibration.UnitAbbrev & _
                            ".)"

                        label_CalibUnitsCenterY.Text = _
                            "(" & _
                            ThisApplication.ActiveImage.SpatialCalibration.UnitAbbrev & _
                            ".)"

                        label_CalibUnitsWidth.Text = _
                            "(" & _
                            ThisApplication.ActiveImage.SpatialCalibration.UnitAbbrev & _
                            ".)"

                        label_CalibUnitsHeight.Text = _
                            "(" & _
                            ThisApplication.ActiveImage.SpatialCalibration.UnitAbbrev & _
                            ".)"

                    Else

                        label_CalibUnitsArea.Text = _
                            "(sq. " & _
                            "pix" & _
                            ".)

                        label_CalibUnitsCenterX.Text = _
                            "(" & _
                            "pix" & _
                            ".)"

                        label_CalibUnitsCenterY.Text = _
                            "(" & _
                            "pix" & _
                            ".)"

                        label_CalibUnitsWidth.Text = _
                            "(" & _
                            "pix" & _
                            ".)"

                        label_CalibUnitsHeight.Text = _
                            "(" & _
                            "pix" & _
                            ".)"

                    End If
            Else

                label_CalibUnitsArea.Text = _
                    ""

                label_CalibUnitsCenterX.Text = _
                    ""

                label_CalibUnitsCenterY.Text = _
                    ""

                label_CalibUnitsWidth.Text = _
                    ""

                label_CalibUnitsHeight.Text = _
                    ""

            End If


    End Sub


    Private Sub timer1_Tick(ByVal sender As System.Object,ByVal e As System.EventArgs) Handles timer1.Tick

        'Call the function that updates the dialog box
        Update_Dialog

    End Sub


End Class
